﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Transformations {
  [Item("ExponentationTransformation", "Represents a exponentation transformation.")]
  public class ExponentiationTransformation : Transformation<double> {
    protected const string BaseParameterName = "Base";

    #region Parameters
    public IValueParameter<DoubleValue> BaseParameter {
      get { return (IValueParameter<DoubleValue>)Parameters[BaseParameterName]; }
    }
    #endregion

    #region properties
    public double Base {
      get { return BaseParameter.Value.Value; }
    }
    #endregion

    [StorableConstructor]
    protected ExponentiationTransformation(bool deserializing) : base(deserializing) { }
    protected ExponentiationTransformation(Transformation<double> original, Cloner cloner)
      : base(original, cloner) {
    }
    public ExponentiationTransformation(IEnumerable<string> allowedColumns)
      : base(allowedColumns) {
      Parameters.Add(new ValueParameter<DoubleValue>(BaseParameterName, "Base for Exponentation", new DoubleValue(Math.E)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExponentiationTransformation(this, cloner);
    }

    public override IEnumerable<double> Apply(IEnumerable<double> data) {
      foreach (double i in data) {
        yield return Math.Pow(Base, i);
      }
    }

    public override bool Check(IEnumerable<double> data, out string errorMsg) {
      errorMsg = "";
      return true;
    }

  }
}
